﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace UnimplementedFormGenerics
{
    public static class MyExtensions
    {
        public class IMyGrouping<TKey, T> : IEnumerable<T>
        {
            public TKey Key { get; private set; }

            private IEnumerable<T> _values;

            public IEnumerator<T> GetEnumerator()
            {
                return _values.GetEnumerator();
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                return _values.GetEnumerator();
            }

            public IMyGrouping(TKey key, IEnumerable<T> values)
            {
                Key = key;
                _values = values;
            }
        }
        // Works:
        // public static IEnumerable<IMyGrouping<TKey, T>> MyGroupBy<TKey, T>(this IEnumerable<T> items, Func<T, TKey> groupByFunc)
        // Error:
        public static IEnumerable<IMyGrouping<TKey, T>> MyGroupBy<T, TKey>(this IEnumerable<T> items, Func<T, TKey> groupByFunc)
        {
            Dictionary<TKey, int> keyToIndex = new Dictionary<TKey, int>();
            List<List<T>> values = new List<List<T>>();
            foreach (var item in items)
            {
                TKey key = groupByFunc(item);
                int idx;
                if (!keyToIndex.TryGetValue(key, out idx))
                {
                    keyToIndex[key] = values.Count;
                    values.Add(new List<T>() { item });
                }
                else
                {
                    values[idx].Add(item);
                }
            }
            return keyToIndex.Select(o => new IMyGrouping<TKey, T>(o.Key, values[o.Value]));
        }
    }

    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            List<string> infos = new List<string>();
            infos.Add("test");
            infos.Add("test");
            foreach (var env in infos.MyGroupBy(o => o))
            {
                mainGrid.Children.Add(new TextBlock() { Text = env.Key });
            }
        }
    }
}
